/*
    Copyright (C) 2001 Hotwarez LLC, Goldtree Enterprises
  
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; 
    version 2 of the License.
  
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
  
    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <stdio.h>
#include <allegro/file.h>
#include "level.h"

extern char g_DataPath[255];

/* Read a level, which is a binary dump of struct level_type on i386
   from disk, do this in an arch independent way.
   
   Note we use the same brillaint error handling as the original fread
   call: none :) */
void read_level(level_type *level, PACKFILE *fp)
{
   union {
      int i;
      float f;
   } u;
   int i, j;

   /* read all the strings at the beginning of the struct */
   pack_fread(level, 9 * 40, fp);

   /* read boolean wire_tube */
   pack_fread(&level->wire_tube, 1, fp);
   
   /* skip 3 bytes of padding */
   pack_fseek(fp, 3);

   /* read the other members of the struct */
   u.i = pack_igetl(fp);
   level->angular_friction = u.f;

   u.i = pack_igetl(fp);
   level->surface_friction = u.f;

   u.i = pack_igetl(fp);
   level->air_friction = u.f;

   level->yon_clipping_plane = pack_igetl(fp);

   /* Orientation contains 9 floats, treat it as an array */
   for (i = 0; i < 2; i++) {
      float *f = (float *)(&level->base_orientations[i]);
      for (j = 0; j < 9; j++) {
         u.i = pack_igetl(fp);
         f[j] = u.f;
      }
   }
   
   for (i = 0; i < 2; i++) {
      float *f = (float *)(&level->turret_orientations[i]);
      for (j = 0; j < 9; j++) {
         u.i = pack_igetl(fp);
         f[j] = u.f;
      }
   }

   for (i = 0; i < 6; i++) {
      float *f = (float *)(&level->vehicle_orientations[i]);
      for (j = 0; j < 9; j++) {
         u.i = pack_igetl(fp);
         f[j] = u.f;
      }
   }

   for (i = 0; i < NUM_GRADIENTS; i++) {
      level->color_info.gradient[i].active = pack_getc(fp);
      level->color_info.gradient[i].first = pack_getc(fp);
      level->color_info.gradient[i].last = pack_getc(fp);
      /* skip 1 byte of padding */ 
      pack_fseek(fp, 1);
      level->color_info.gradient[i].num_colors = pack_igetl(fp);
   }
   level->color_info.size = pack_igetl(fp);
}


int Load_Level( char *filename, level_type *level )
    {
     PACKFILE *fp;
	char newfilename[512];

	sprintf(newfilename,"%s%s",g_DataPath,filename);


     if( (fp = pack_fopen( newfilename, "r" )) == NULL )
         return 0;

     read_level(level, fp);

     pack_fclose( fp );
    
     return(1); /* Function was successful */
    }
